;#######################################
;# Config.h for MLC2 version 0.3       #
;#######################################

;This is the config file for the MLC2 sources
;Here you can define the sizes of buffers and the
;handling of certain situations that do not have a 
;common solution. You may edit this file by hand or
;use the provided program CEdit.exe (Windows PE File)
;to manage the file. The later is recommended since it
;watches for the buffer sizes so that the calcs memory
;will be enough to run the program

;The Size of the Program. Please do ONLY change this if you
;edit the sources since the CEdit program relies on this size
;to determine the progams memory usage.

ProgramSize = 36015

;#Option:	Config_Version	
;#Values:	<Integer>
;#Description:	ensures that only matching config files are used with the program.
;		Set this to the programs version (without the dot)

Config_Version = 04

;#Option:	Exit_Method
;#Values: 	emReboot/emTerminate
;#Size:		emTerminate=-0
;#Description:	This is used to dertermine what the program does when it exits.
;		Since the normal exit (emTerminate) is still somewhat buggy rebooting
;		(emReboot) is recommended

Exit_Method equ emReboot

;#Option:	Instant_Execution	
;#Values: 	No/Yes
;#Size:		Yes=-240
;#Description:	This determines if there are messages and information displayed before
;		the real execution starts (No) or if the programm is started instantly (Yes)
;		For developers the first method is recommended since it shows them various
;		status information of the virtual machine
;		This does also prevent the program from showing its setup status for error handling

Instant_Execution equ No

;#Option:	Instant_Exit
;#Values:	No/Yes
;#Size:		Yes=-164
;#Description:	Should the program display debug information before exit  (No)
;		(e.g. number of executed lines) or should it just terminate without further user
;		input (Yes) The later is recommended for final programs that just want to
;		exit to the calc and take care of the exit themselves / are debugged.


Instant_Exit equ No

;#Option:	Memory_Method
;#Values:	Independant/Shared
;#Size:		Shared=-oStack_Size-aStack_Size
;#Description:	The shared memory method uses the same buffers for the tables at compile time as for
;		the stacks at interpretion (runtime). The independant method uses the same buffers.
;		The shared method however, binds the oStack size to the variable table and the aStack size
;		to the address table resulting an a easy and cheap increase of stack sizes without using
;		additional memory but with sacrificing the flexibility and scalability of the sizes.

Memory_Method equ mmIndependant

;#Option:	Variable_Table_Size
;#Values:	<integer>
;#Size:		Value=+Variable_Table_Size+Variable_Table_Size+Variable_Table_Size+Variable_Table_Size
;#Description:	This is the table where the memory addresses of the variables and the pointers to their names
;		are held in. The number is given as a double word meaning an increase of 1 results in a size increase
;		of 4. If locked to the Stacks an increase of 1 also means an increase of 2 for the Operand_Stack_Size

Variable_Table_Size = 512

;#Option:	Address_Table_Size
;#Values:	<integer>
;#Size:		Value=+Address_Table_Size+Address_Table_Size+Address_Table_Size+Address_Table_Size
;#Description:	This is the table where the compiler holds the pointers to variables memory addresses and
;		the pointer to their text name. Each value is 4 bytes long and can hold 1 variable.
;		So if you want to use 10000 Variables, what is quite unlikely you must set this to 10000
;		I think the standard value of 4096 is safe and should also be safe for the address stack if
;		it is linked to this table.

Address_Table_Size = 4096

;#Option:	Operand_Stack_Size
;#Depends:	Memory_Method=Independant
;#Values:	<integer>
;#Size:		Value=+Operand_Stack_Size+Operand_Stack_Size
;#Description:	The stack where the interpreter holds operands for commands like add or sub
;		and similar things. If this value is independant then it may be set. Else it
;		is equal to the Variable_Table_Size*2. Values are words, so every added word results
;		in 2 byes added to the bytecode. The initial value of 256 should be more than sufficient,
;		even for bigger, nested loops!
Operand_Stack_Size = 256

;#Option:	Address_Stack_Size
;#Depends:	Memory_Method=Independant
;#Values:	<integer>
;#Size:		Value=+Address_Stack_Size+Address_Stack_Size
;#Description:	The stack where the interpreter holds return addresses for nested functions and their parameters.
;		If this value is independant then it may be set. Else it is equal to the Address_Table_Size*2. 
;		Values are words, so every added word results in 2 byes added to the bytecode. 
;		Here it is valid, the bigger, the better! More size means more nested loops. Every nested loop
;		Without parameters needs 2 words on this stack. Every parameter adds another 2 words. So if
;		you need 100 nested loops with 10 parameters each you should set this to 4000. If it is shared
;		then you may not set a value. However, please look that the Address Table is big enough since this
;		is the shared memory for the address stack.

Address_Stack_Size = 256

;#Option:	Buffered_Draw
;#Values:	No/Yes
;#Size:		<unknown>
;#Description:	Determines if the screen is always drawn immediately or if it is handled like in MLC2 where the
;		Screen is only drawn after the #DRAW command has been issued.
;		This option is for testing only. When the #DRAW command is encountered during execution on "Non-draw"
;		interpreters it is just ignored and all graphics operations go straight to the screen.

Buffered_Draw equ Yes

ByteCode_Size = 1024	;In Byte
Data_Size = 1024		;In Word
Extra_Size = 512	;In Word
String_Size = 2048	;in Byte

;#EPS
;#Please do not edit below! These settings must they as they are
;#For proper operation of the program!

;Value defines:
No 		equ 0
Off 		equ 0
On 		equ 1
Yes 		equ 1
emReboot 	equ 2
emTerminate 	equ 3
mmIndependant	equ 4
mmShared	equ 5

IF Memory_Method eq mmShared
	Operand_Stack_Size = Variable_Table_Size*2	
	Address_stack_Size = Address_Table_Size*2
ENDIF

