;########################################################
;#         Screen  I/O-routines                         #
;########################################################
;#      Date of Creation: 20-10-2005                    #
;########################################################
;#      Laste changed:  02-01-2005                      #
;#                                                      #
;#Changelog:                                            #
;#      21-10-05        Added String Escape capabilities#
;#      23-10-05        Added Graphics library          #
;#      24-10-05        Fixed scrolling bug, graphics   #
;#	06-12-05	Added Graphic scrolling pro-type#
;#	02-01-06	Renamed to reflect contents	#
;#	16-05-06	Added Hex string printing	#
;#                                                      #
;########################################################
;
;This file contains all commands that are related to text based in- and
;output. All end-user routines are used for text based as well as for
;graphics based interaction. 
;Additionally the file contains some constants which are used to define
;the state of the System.
;
;########################################
;#              Constants               #
;########################################

;#### IO-Types
ioGraphic	equ 000h	;For text based output (only used till graphics init)
ioText		equ 001h	;For graphics based output used for all MLC-L scripts
ioScreenSave	equ 0FFh	;Set up if the screen saver is on. Prevents calling of
				;window/screen refresh and so on

;# TextFlags #
txtUnderlined	equ 001h
txtBold 	equ 002h
txtItalics	equ 004h

;# Graphics flags
psOn		equ 01h
psOff		equ 00h

;# Colors

clInverse	equ 10
clBlack 	equ 8
clTransparent	equ 11
clWhite		equ 0

;#### state definition variables

ioState db 0			;Used to define the state of the machine (see IO-Types)
Cursor	dw 00			;Cursor position
pageSwitch db 0 		;is set to 1 if grayscale enabled
narrow db 0
LastColor db clBlack

;#Includes:
include inc\graphics.mac
;# Private routines #


;This routine draws a line and relies on the SetPixel procedure
;In: x1,y1 = DH/DL, x2,y2 = BH/BL, AH = Color
;

steep db 0

deltax 		db 0
deltay	 	db 0	;=Deltaerr
Error 		db 0
X		db 0
Y 		db 0
LineColor 		db 0

_line proc
	;x0/y0 = BX, x1/y1 = CX, AH = Color
	MOV byte ptr CS:[steep],0			;Assume non-steep line
	MOV byte ptr CS:[Linecolor],AH			;And read the line color from ah
	PUSH BX CX DX AX				;Save the registers
	MOV AL,BH
	MOV AH,CH
	call _abssub					;do ABS(x0-x1) --> AL
	MOV DL,AL					;result in DL now
	MOV AL,BL
	MOV AH,CL
	call _abssub					;do ABS(y0-y1) --> AL
	CMP AL,DL					;Is y difference bigger than x difference
	JNA @@continue					;if not go ahead
	
	XCHG BH,BL					;else exchange the x and y values ...
	XCHG CH,CL					;Switch it if it is steep
	MOV byte ptr CS:[steep],1
	@@continue:					;So now we are independant of the steepness until plot ...

	MOV AL,BH
	MOV AH,CH
	call _abssub					;do ABS(x0-x1) --> AL
	MOV byte ptr CS:[Deltax],AL			;result in deltax now
	MOV AL,BL
	MOV AH,CL
	call _abssub					;do ABS(y0-y1) --> AL
	MOV byte ptr CS:[Deltay],AL			;result in deltay now
	MOV byte ptr CS:[ERror],0			;We do not yet need to update y

	CMP BH,CH
	JB @@no_minus
	MOV DH,-1
	JMP @@minus
	@@no_minus:	
	MOV DH,1		;xstep
@@minus:

	CMP BL,CL
	JB @@no_minus1
	MOV DL,-1
	JMP @@minus1
	@@no_minus1:
	MOV DL,1		;yStep
	@@minus1:

	CMP byte ptr CS:[steep],1
	JE @@plotyx
	JNE @@plotxy

@@next1:
		;Now the loop starts
	CMP BH,CH
	JE @@ende
	ADD BH,DH	;Add xstep
	MOV AL,CS:[Deltay]
	ADD CS:[ERROR],AL
	MOV AL,CS:[Error]
	CMP AL,CS:[DeltaX]
	JE @@handleError
	@@returner:
	CMP CS:[steep],1
	JE @@plotyx
	JNE @@plotxy
		

@@ende:
	POP AX DX CX BX
	ret
	@@plotxy: 
		PUSH DX
			MOV DX,BX
			MOV AH,byte ptr CS:[Linecolor]
			call _setPixel
		POP DX 
		JMP @@next1
	@@plotyx:
		PUSH DX
			MOV DL,BH
			MOV DH,BL
			MOV AH,byte ptr CS:[Linecolor]
			call _setPixel
		POP DX
		JMP @@next1
	@@handleERror:
		ADD BL,DL
		MOV AL,CS:[DeltaX]
		SUB CS:[ERror],AL
		JMP @@returner
endp

_Rect proc
	;BX = x0,y0
	;CX = x1,y1
	;AH = Line Color
	;AL = Fill Color
	PUSH CX
	MOV CL,BL
	call _Line
	POP CX

	PUSH CX
	MOV CH,BH
	call _Line
	POP CX

	PUSH BX
	MOV BL,CL
	call _Line
	POP BX
	PUSH BX

	MOV BH,CH
	call _Line
	POP BX

	CMP AL,clTransparent
	JE @@ende

	PUSH BX CX DX

;Swap to right format
	CMP BH,CH
	JB @@NswapH
	XCHG BH,CH
	@@NSwapH:
	CMP BL,CL
	JB @@NSwapL
	XCHG BL,CL
	@@NSwapL:

	ADD BX,0101h
	MOV DX,BX

	MOV AH,AL

	@@repeat: call _SetPixel	
	INC DH
	CMP DH,CH
	JB @@repeat
	MOV DH,BH
	INC DL
	CMP DL,CL
	JB @@repeat
	POP DX CX BX
	@@ende: 
	ret
endp

_abssub proc
	;do AH-AL or AL-AH (so that a value > 0 is the result) and returns it in AL
	CMP AH,AL
	JB @@other
	SUB AH,AL
	MOV AL,AH
	ret
	@@other:
		SUB AL,AH
	ret
endp




;The routine to set pixels ...
;In: Position at DH/DL, Color=AH
;
	
	
_SetPixel proc
	PUSH ES BX SI CX
	PUSH 01A20h
	POP ES
	MOV BX,04080h
	SUB BH,DL
	SUB BL,DH
	MOV SI,BX		;y=BH
	SHR SI,8		;Only use the high byte, but as low byte.
	SHL SI,4		;Adjust to line
	XOR BH,BH
	MOV CL,BL
	SHR BL,3
	AND CL,7
	ADD SI,BX
	db 00Fh,014h,0C7h	;Set1 BH,CL

	CMP AH,clWhite 
	JE @@reset
	CMP AH,clInverse
	JE @@inverse

	OR ES:[SI],BH		;Write
	;//TODO Insert paging algorithm here!	

	@@ende: POP CX SI BX ES
		ret
	@@reset:
		NOT BH
		AND ES:[SI],BH
		JMP @@ende
	@@inverse:
		XOR ES:[SI],BH
		JMP @@ende
endp

;Reads 1 pixel from the screen
;In: DL/DH = Position
;Out: AH = COlor

_GetPixel proc
	PUSH CX SI BX ES
	PUSH 01A20h
	POP ES

	MOV BX,04080h
	SUB BH,DH
	SUB BL,DL

	
	MOV SI,BX		;y
	SHR SI,8		;Only use the high byte but use it as low ..
	SHL SI,4		;Adjust to line
	XOR BH,BH
	MOV CL,BL		;x
	SHR BL,3
	AND CL,7
	ADD SI,BX
	db 00Fh,014h,0C7h
	XOR AH,AH

	@@looper:
	AND BH,ES:[SI]		;Is set?
	JZ @@next
	SHL AH,1
	INC AH
	@@next: 
	ADD SI,1024
	CMP SI,4095
	JNA @@looper

	POP ES BX SI CX
	ret
endp

;################################################
;#              The Print Char routine          #
;################################################
;# Print the Char in AL at the position DH/DL   #
;# the color is defined in AH                   #
;# More flags are taken from the txtState var   #
;################################################
_SetGraphicChar proc
	PUSH ES DI AX BX DX CX DS SI
	MOV AH,CS:[LastColor]
	XCHG DH,DL		;Improve!!!
	XOR BX,BX
	MOV BL,DH
	SHL BX,4		;*16
	MOV DH,DL		;for rest
	SHR DL,3		;/8
	AND DH,7		;rest in DH     ... this is the start pixel

	PUSH DX
	XOR DH,DH
	ADD BX,DX		;Byte of position is in BX
	pOP DX

	MOV DI,1024
	SUB DI,BX		;now discard BX ...
	PUSH 1A20h		;Screen segment
	POP ES
	XOR BX,BX
	MOV SI,offset CS:[font1]
	MOV BL,AL
	SHL BX,3
	PUSH AX
	MOV AH,0
	SUB Bx,AX
	SUB BX,AX
	POP AX
	ADD SI,BX
	MOV CX,6
      @@charsetter:
	XOR BX,BX
	MOV BL,byte ptr CS:[SI]
	SHL BX,011
	XCHG CL,DH
	SHR BX,CL
	XCHG CL,DH
	CMP AH,clTransparent
	JE @@done
	CMP AH,clBlack
	JNE @@next
	OR ES:[DI-2],BX
	JMP @@done
@@next:	CMP AH,clInverse
	JNE @@next2
	XOR ES:[DI-2],BX
	JMP @@done
@@next2: ;White ... 
	NOT BX
	AND ES:[DI-2],BX


      @@done: SUB DI,16
	INC SI
	Loop @@charsetter
	
	POP SI DS CX DX BX AX DI ES
	ret
endp


;################################################
;#The Print Char routine for text based actions #
;################################################
;# Print the Char in AL at the position DH/DL   #
;################################################

_SetTextChar proc
	PUSH AX BX CX	DX;Save all the registers that are changed

	XCHG DL,DH


	MOV AH,02h	;Set the cursor
	XOR BX,BX	;of page with no flags
	INT 10h 	;now

	MOV AH,09h	;Print Char routine
	MOV CX,1	;Print Char once
	INT 10h 	;and call it

	POP DX CX BX AX	;And now restore the changed registers
	ret		;and return
endp

;# Public routines #

;################################################
;#      The print routine                       #
;################################################
;# IN: DS:SI    =       Pointer to string       #
;#	%h print 4 digit hex string		#
;#      %n in string is replaced with Word in BX#
;#      There are always 5 chars inserted       #
;#      %c in string is replaced with byte in BL#
;#              There are always 3 chars        #
;#      %% in string is replaced with the % char#
;#      %0-%9 in string changes font color      #
;#      %u in string underlines the text        #
;#      %U end of underlining                   #
;#      %b prints the text bold                 #
;#      %B end bold printing                    #
;#      %i Print the text italics               #
;#      %I End italics printing                 #
;#      %t Print the text transparent (no bg)   #
;#      %T End transparent printing             #
;#      %s Switch colors (print inverse)        #
;#      %S End switching colors                 #
;#      %r new Line                             #
;#                                              #
;# all escape sequences are obviously case      #
;# sensitive.                                   #
;# All escape chars except %c, %n, %r and %% are#
;# ignored in text based mode for the text to   #
;# remain readable.                             #
;#                                              #
;################################################
;# Quick Hack: if CS:[Narrow] = 1, spacing = 0

_setChar proc
	CMP ioState,ioGraphic
	JE @@graph
	call _settextChar
	INC DH
	ret
	@@graph:
	call _setGraphicChar
	ADD DH,6
	SUB DH,CS:[narrow]
	ret
endp

_Print proc
	PUSH AX DX
	MOV AH,byte ptr CS:[LastColor]
     @@PrintLoopCursor:
	MOV DX,CS:[cursor]	;Set the cursor from the saved position
     @@PrintLoop:
	LODSB		;Load the char
	CMP AL,'%'	;Is escape char?
	JE @@escape	;Then jump to handling
	CMP AL,0	;String is finished then exit procedure
	JE @@ende	
	@@setTExt:
	call _setChar
	JMP @@PrintLoop 	;Then get the next char
     @@move_cursor_down:	;Move the cursor down (%r / return sequence)
	INC DL			;First increase the cursor
	XOR DH,DH		;and set the cursor to first column
	CMP ioState,ioGraphic	;When in graphics mode change movement
	JE @@graph_mov
	CMP DL,8		;Then test if there is scrolling needed
	JNE @@PrintLoop 	;If not then the show can go on.
	DEC DH			;Else decrease DH again 
				;Screen movement is broken!
	call _ScrollUp
	JMP @@PrintLoop 	;And continue printing the text
     @@escape:
	MOV CS:[cursor],DX
	LODSB
	CMP AL,0		;If end then ignore escape char and exit
	JE @@ende
	CMP AL,'r'		;If return then
	JE @@move_cursor_down	;Go to move cursor down routine
	CMP AL,'%'		;If % sign then
	JE @@setText		;Print the % sign :D
	CMP AL,'c'		;If print byte in BL then do so ...
	JE @@PrintByte
	CMP AL,'h'
	JE @@printHex
	CMP AL,'n'
	JE @@PrintWord
	CMP AL,'0'
	JB @@nextTest
	CMP AL,'8'
	JA @@NextTest
	JMP @@PrintColor
@@NextTest:
	CMP AL,'s'
	JE @@PrintInverse
	MOV DX,CS:[cursor]
	JMP @@printLoop 	;Unknown Escape sequence! Ignore it ;)
     @@graph_mov:
		ADD DL,6
		CMP DL,58
		JB @@printLoop
;		call _ScrollUp
		JMP @@printLoop
     @@PrintByte:
	call _PrintByte
	JMP @@printLoopCursor

     @@PrintWord:
	call _PrintWord
	JMP @@printLoopCursor
     @@PrintColor:
	MOV AH,AL
	SUB AH,48	;Number is set ...
	MOV CS:[LastColor],AH

	JMP @@printLoopCursor
     @@printHex:
	call _PrintHex
	JMP @@printLoopCursor
     @@PrintInverse:
	MOV CS:[LastColor],clInverse
	JMP @@printLoopCursor

     @@ende:
	MOV CS:[cursor],DX	;Save the cursor for later use
	MOV byte ptr CS:[LastColor],AH
	POP DX AX		;and restore the registers save at the beginning
	ret			;then return
endp

;################################################
;#	Print a command (for debugger)		#
;################################################

cmdBuffer db '        ',0
cmdPushi8 db 'PUSHi8 %c',0
cmdPushm8 db 'Pushm8 %c',0
cmdPushi16 db 'PUSHi16 %n',0
cmdPushm16 db 'Pushm16 %n',0
cmdPopm8  db 'POPm8 %c',0
cmdLDS    db 'LDS %c',0
cmdPopm16    db 'POPm16 %n',0
cmdPOPmES16 db 'POPmE16 %n',0
cmdCLRSTR db 'CLRSTR %n',0

_PrintCMD proc
	PUSH SI DS CX ES
	;Command in BX
	MOV SI,offset CS:[DebugTable]

	PUSH seg BC_Target
	POP DS

	
	MOV CL,BL		;Fix with native code (TestBit 8)
	AND CL,080h
	CMP CL,080h
	JNE @@nodo_pushi8
		JMP @@do_pushi8
	@@nodo_Pushi8:

	CMP BL,035h
	JNE @@no_Pushm8
  	  MOV BL,DS:[DI+1]
	  print cmdPushm8
	  JMP @@ok
      @@no_pushm8:
	CMP BL,04Ch
	JNE @@no_CLRSTR
	MOV BX,word ptr DS:[DI+1]
	print cmdCLRSTR
	JMP @@incDIjmp
      @@no_CLRSTR:
	CMP BL,03Ah
	JNE @@no_popm8
	  MOV BL,DS:[DI+1]
 	  print cmdPopm8
	  JMP @@ok
      @@no_popm8:
	CMP BL,045h
	JE @@do_LDS
	CMP BL,034h
	JE @@do_pushi16
	CMP BL,036h
	JNE @@nodo_pushm16
		JMP @@do_pushm16
	@@nodo_pushm16:
	CMP BL,04Ah
	JNE @@nodo_popmes16
		JMP @@do_popmES16
	@@nodo_popmES16:
	CMP BL,03Bh
	JNE @@noSpecial


		MOV BX,DS:[DI+1]
		print cmdPopm16
		JMP @@incDIJmp

      @@noSpecial:

	XOR BH,BH
	SHL BX,3
	ADD SI,BX
	
	PUSH DI
	MOV DI,offset cmdBuffer
	PUSH seg cmdBuffer
	POP ES
	PUSH seg DebugTable
	POP DS
	MOV CX,4
	REP MOVSW
	POP DI

	Print cmdBuffer

	DEC DI
	@@ok: 
	INC DI
	POP ES CX DS SI
	ret	
@@do_Pushi8:
	AND BL,7Fh
	print cmdPushi8
	DEC DI
	JMP @@ok
@@do_LDS:
	MOV BL,DS:[DI+1]
	print cmdLds
	JMP @@ok
@@do_Pushi16:
	MOV BX,DS:[DI+1]
	print cmdPushi16
	JMP @@incDIJmp
@@do_Pushm16:
	MOV BX,DS:[DI+1]
	print cmdPushm16
	JMP @@incDIJmp
@@do_PopmES16:
	MOV BX,DS:[DI+1]
	print cmdPopmES16
@@incDIJmp:
	INC DI
	JMP @@ok
endp

;################################################
;#      The print byte routine                  #
;################################################
;# BX   =       Word to print                   #
;#                                              #
;# If the carry flag is set then the text is    #
;# printed inverse, else it is printed notmally #
;################################################

_PrintWord proc
	PUSH AX DX BX

	MOV AL,'0'
	MOV DX,CS:[cursor]
	CMP BX,9999
	JA @@print5
	call _SetChar
	CMP BX,999
	JA @@print4
	call _SetChar
	CMP BX,99
	JA @@print3
	call _SetChar
	CMP BX,9
	JA @@print2
	call _setChar
	CMP BX,0
	JNE @@print1
	call _setChar
	jmp @@ende
	@@print5:
		MOV AX,BX
		MOV BX,10000
		PUSH DX
		XOR DX,DX
		DIV BX
		MOV BX,DX
		POP DX
		add AL,48
		call _setChar
	@@print4:
		MOV AX,BX
		MOV BX,1000
		PUSH DX
		XOR DX,DX
		DIV BX
		MOV BX,DX
		POP DX
		add AL,48
		call _setChar
	@@print3:
		MOV AX,BX
		MOV BX,100
		PUSH DX
		XOR DX,DX
		DIV BX
		MOV BX,DX
		POP DX
		add AL,48
		call _setChar
	@@print2:
		XOR Ax,AX
		MOV AL,BL
		MOV BH,10
		PUSH DX
		DIV BH
		POP DX
		MOV BL,AH
		add AL,48
		call _setChar
	@@print1:
		MOV AL,BL
		ADD AL,48
		call _setChar
@@ende:		
		
	MOV CS:[cursor],DX
	POP BX DX AX
	ret
endp

_PrintHex proc
	PUSH AX DX BX

	MOV AL,'0'
	MOV DX,CS:[cursor]
	CMP BX,0FFFh
	JA @@print4
	call _SetChar
	CMP BX,0FFh
	JA @@print3
	call _SetChar
	CMP BX,0Fh
	JA @@print2
	call _setChar
	CMP BX,0
	JNE @@print1
	call _setChar
	jmp @@ende
	@@print4:
		MOV AX,BX
		SHR AX,12
		add AL,48
		CMP AL,57
		JNA @@noHex
		ADD AL,7
		@@noHex:
		call _setChar
	@@print3:
		MOV AX,BX
		SHL AX,4
		SHR AX,12
		add AL,48
		CMP AL,57
		JNA @@noHex2
		ADD AL,7
		@@noHex2:
		call _setChar
	@@print2:
		MOV AL,BL
		SHR AL,4
		add AL,48
		CMP AL,57
		JNA @@noHex3
		ADD AL,7
		@@noHex3:
		call _setChar
	@@print1:
		MOV AL,BL
		AND AL,0Fh
		add AL,48
		CMP AL,57
		JNA @@noHex4
		ADD AL,7
		@@noHex4:
		call _setChar
@@ende:		
		
	MOV CS:[cursor],DX
	POP BX DX AX
	ret
endp


_PrintByte proc
	PUSH AX DX BX

	MOV DX,CS:[cursor]
	CMP BL,99
	JA @@print3
	MOV AL,'0'
	call _SetChar
	CMP BL,9
	JA @@print2
	call _setChar
	CMP BL,0
	JNE @@print1
	call _setChar
	jmp @@ende
	@@print3:
		XOR Ax,AX
		MOV AL,BL
		MOV BH,100
		PUSH DX
		DIV BH
		POP DX
		MOV BL,AH
		add AL,48
		call _setChar
	@@print2:
		XOR Ax,AX
		MOV AL,BL
		MOV BH,10
		PUSH DX
		DIV BH
		POP DX
		MOV BL,AH
		add AL,48
		call _setChar
	@@print1:
		MOV AL,BL
		ADD AL,48
		call _setChar
		
	
@@ende: MOV CS:[cursor],DX
	POP BX DX AX
	ret
endp

_InitText proc			;Initialize the text based routines
	MOV CS:[cursor],0	;To do this you set the cursor to the origin of the screen
	MOV CS:[ioState],ioText ;Set state of machine to text
	PUSH AX
	MOV AL,0C3h
	OUT 02,AL
	POP AX
	Cls
	Switch psOff
	ret
endp

_InitGraphics proc
	MOV CS:[cursor],0	;Reset the cursor position
	MOV CS:[ioState],ioGraphic
	PUSH AX
	MOV AL,0D3h
	OUT 02,AL
	POP AX
	Cls
	Switch psOn
	ret
endp

_Cls proc
	PUSH AX CX ES DI	;Save the registers
	XOR AX,AX		;and clear
	MOV CX,512*4		;the screen
	PUSH 1A20h		;by writing with
	POP ES			;0s
	XOR DI,DI		;512
	REP STOSW		;times now
	MOV word ptr CS:[cursor],0
	POP DI ES CX AX 	;then resore registers.
	ret
endp

_scrollUp proc
	PUSH ES DI DS SI AX CX
	MOV AX,01A20h		;Set addresses of video buffe
	MOV ES,AX
	MOV DS,AX
	MOV DI,63
	MOV SI,55
	XOR AX,AX
	STD			;Move reversen since we start in lower right corner
	@@movecol:
	MOV CX,28		
	REP MOVSW		;And move the fist 7 lines
	MOV CX,4
	REP STOSW		;and then clear the last line
	ADD DI,128
	ADD SI,120
	CMP SI,16*64		;Do this until all columns are moved up
	JBE @@movecol
	CLD			;And DON'T forgett to reset the direction ;)
	POP CX AX SI DS DI ES	;Then restore th registers
	ret			;and return to caller
endp