;########################################################
;#      Text based I/O-routines                         #
;########################################################
;#      Date of Creation: 20-10-2005                    #
;########################################################
;#      Laste changed:  24-10-2005                      #
;#                                                      #
;#Changelog:                                            #
;#      21-10-05        Added String Escape capabilities#
;#      23-10-05        Added Graphics library          #
;#      24-10-05        Fixed scrolling bug, graphics   #
;#	06-12-05	Added Graphic scrolling pro-type#
;#                                                      #
;########################################################
;
;This file contains all commands that are related to text based in- and
;output. All end-user routines are used for text based as well as for
;graphics based interaction. 
;Additionally the file contains some constants which are used to define
;the state of the System.
;
;########################################
;#              Constants               #
;########################################

;#### IO-Types
ioGraphic	equ 000h	;For text based output (only used till graphics init)
ioText		equ 001h	;For graphics based output used for all MLC-L scripts
ioScreenSave	equ 0FFh	;Set up if the screen saver is on. Prevents calling of
				;window/screen refresh and so on

;# TextFlags #
txtUnderlined	equ 001h
txtBold 	equ 002h
txtItalics	equ 004h
txtTransparent	equ 008h
txtInverse	equ 010h

;# Graphics flags
psOn		equ 01h
psOff		equ 00h

;#### state definition variables

ioState db 0			;Used to define the state of the machine (see IO-Types)
Cursor	dw 00			;Cursor position
Color	db 0			;is set to 7 each time a print routine is called. No memory effect
pageSwitch db 0 		;is set to 1 if grayscale enabled


;#Includes:
include inc\text_prc.mac
;# Private routines #

;################################################
;#              The Print Char routine          #
;################################################
;# Print the Char in AL at the position DL/DH   #
;# the color is defined in AH                   #
;# More flags are taken from the txtState var   #
;################################################

_SetGraphicChar proc
	;The graphics routines have not yet been written this will be done later.
	PUSH ES DI AX BX DX CX SI

	XOR BX,BX
	MOV BL,DH
	SHL BX,4		;*16
	MOV DH,DL		;for rest
	SHR DL,3		;/8
	AND DH,7		;rest in DH     ... this is the start pixel

	PUSH DX
	XOR DH,DH
	ADD BX,DX		;Byte of position is in BX
	pOP DX

	MOV DI,1024
	SUB DI,BX		;now discard BX ...
	PUSH 1A20h		;Screen segment
	POP ES
	XOR BX,BX
	MOV SI,offset CS:[font1]
	MOV BL,AL
	SHL BX,3
	PUSH AX
	MOV AH,0
	SUB Bx,AX
	SUB BX,AX
	POP AX
	ADD SI,BX
	MOV CX,6
      @@charsetter:
	XOR BX,BX
	MOV BL,byte ptr CS:[SI]
	SHL BX,011
	XCHG CL,DH
	SHR BX,CL
	XCHG CL,DH
	OR ES:[DI-2],BX
	SUB DI,16
	INC SI
	Loop @@charsetter
	
	call _drawMouse 
	POP SI CX DX BX AX DI ES
	ret
endp


;################################################
;#The Print Char routine for text based actions #
;################################################
;# Print the Char in AL at the position DL/DH   #
;################################################

_SetTextChar proc
	PUSH AX BX CX	;Save all the registers that are changed

	MOV AH,02h	;Set the cursor
	XOR BX,BX	;of page with no flags
	INT 10h 	;now

	MOV AH,09h	;Print Char routine
	MOV CX,1	;Print Char once
	INT 10h 	;and call it
	call _drawMouse

	POP CX BX AX	;And now restore the changed registers
	ret		;and return
endp

;# Public routines #

;################################################
;#      The print routine                       #
;################################################
;# IN: DS:SI    =       Pointer to string       #
;#      %n in string is replaced with Word in BX#
;#      There are always 5 chars inserted       #
;#      %c in string is replaced with byte in BL#
;#              There are always 3 chars        #
;#      %% in string is replaced with the % char#
;#      %0-%9 in string changes font color      #
;#      %u in string underlines the text        #
;#      %U end of underlining                   #
;#      %b prints the text bold                 #
;#      %B end bold printing                    #
;#      %i Print the text italics               #
;#      %I End italics printing                 #
;#      %t Print the text transparent (no bg)   #
;#      %T End transparent printing             #
;#      %s Switch colors (print inverse)        #
;#      %S End switching colors                 #
;#      %r new Line                             #
;#                                              #
;# all escape sequences are obviously case      #
;# sensitive.                                   #
;# All escape chars except %c, %n, %r and %% are#
;# ignored in text based mode for the text to   #
;# remain readable.                             #
;#                                              #
;################################################
;# If the carry flag is set then the text is    #
;# printed inverse, else it is printed notmally #
;################################################

_setChar proc
	CMP ioState,ioGraphic
	JE @@graph
	call _settextChar
	INC DL
	ret
	@@graph:
	call _setGraphicChar
	ADD DL,6
	ret
endp

_Print proc
	PUSH AX DX
	MOV DX,cursor	;Set the cursor from the saved position
     @@PrintLoop:
	LODSB		;Load the char
	CMP AL,'%'	;Is escape char?
	JE @@escape	;Then jump to handling
	CMP AL,0	;String is finished then exit procedure
	JE @@ende	
	@@setTExt:
	call _setChar
	JMP @@PrintLoop 	;Then get the next char
     @@move_cursor_down:	;Move the cursor down (%r / return sequence)
	INC DH			;First increase the cursor
	XOR DL,DL		;and set the cursor to first column
	CMP ioState,ioGraphic	;When in graphics mode change movement
	JE @@graph_mov
	CMP DH,8		;Then test if there is scrolling needed
	JNE @@PrintLoop 	;If not then the show can go on.
	DEC DH			;Else decrease DH again 
				;Screen movement is broken!
	call _ScrollUp
	JMP @@PrintLoop 	;And continue printing the text
     @@escape:
	MOV CS:[cursor],DX
	LODSB
	CMP AL,0		;If end then ignore escape char and exit
	JE @@ende
	CMP AL,'r'		;If return then
	JE @@move_cursor_down	;Go to move cursor down routine
	CMP AL,'%'		;If % sign then
	JE @@setText		;Print the % sign :D
	CMP AL,'c'		;If print byte in BL then do so ...
	JE @@PrintByte
	CMP AL,'n'
	JE @@PrintWord
	MOV DX,CS:[cursor]
	JMP @@printLoop 	;Unknown Escape sequence! Ignore it ;)

     @@graph_mov:
		ADD DH,6
		CMP DH,58
		JB @@printLoop
;		call _ScrollUp
		JMP @@printLoop
     @@PrintByte:
	call _PrintByte
	JMP @@printLoop

     @@PrintWord:
	call _PrintWord
	JMP @@printLoop

     @@ende:
	MOV CS:[cursor],DX	;Save the cursor for later use
	POP DX AX		;and restore the registers save at the beginning
	ret			;then return
endp

;################################################
;#      The print byte routine                  #
;################################################
;# BX   =       Word to print                   #
;#                                              #
;# If the carry flag is set then the text is    #
;# printed inverse, else it is printed notmally #
;################################################

_PrintWord proc
	PUSH AX DX BX

	MOV AL,'0'
	MOV DX,CS:[cursor]
	CMP BX,9999
	JA @@print5
	call _SetChar
	CMP BX,999
	JA @@print4
	call _SetChar
	CMP BX,99
	JA @@print3
	call _SetChar
	CMP BX,9
	JA @@print2
	call _setChar
	CMP BX,0
	JNE @@print1
	call _setChar
	jmp @@ende
	@@print5:
		MOV AX,BX
		MOV BX,10000
		PUSH DX
		XOR DX,DX
		DIV BX
		MOV BX,DX
		POP DX
		add AL,48
		call _setChar
	@@print4:
		MOV AX,BX
		MOV BX,1000
		PUSH DX
		XOR DX,DX
		DIV BX
		MOV BX,DX
		POP DX
		add AL,48
		call _setChar
	@@print3:
		XOR AH,AH
		MOV AL,BL
		MOV BH,100
		PUSH DX
		DIV BH
		POP DX
		MOV BL,AH
		add AL,48
		call _setChar
	@@print2:
		XOR Ax,AX
		MOV AL,BL
		MOV BH,10
		PUSH DX
		DIV BH
		POP DX
		MOV BL,AH
		add AL,48
		call _setChar
	@@print1:
		MOV AL,BL
		ADD AL,48
		call _setChar
		
	
@@ende: MOV CS:[cursor],DX
	POP BX DX AX
	ret
endp

_PrintByte proc
	PUSH AX DX BX

	MOV DX,cursor
	CMP BL,99
	JA @@print3
	MOV AL,'0'
	call _SetTextChar
	INC DL
	CMP BL,9
	JA @@print2
	call _setTextChar
	INC DL
	CMP BL,0
	JNE @@print1
	call _setTextChar
	INC DL
	jmp @@ende
	@@print3:
		XOR Ax,AX
		MOV AL,BL
		MOV BH,100
		PUSH DX
		DIV BH
		POP DX
		MOV BL,AH
		add AL,48
		call _setChar
	@@print2:
		XOR Ax,AX
		MOV AL,BL
		MOV BH,10
		PUSH DX
		DIV BH
		POP DX
		MOV BL,AH
		add AL,48
		call _setChar
	@@print1:
		MOV AL,BL
		ADD AL,48
		call _setChar
		
	
@@ende: MOV CS:[cursor],DX
	POP BX DX AX
	ret
endp

_InitText proc			;Initialize the text based routines
	MOV CS:[cursor],0	;To do this you set the cursor to the origin of the screen
	MOV CS:[ioState],ioText ;Set state of machine to text
	PUSH AX
	MOV AL,0C3h
	OUT 02,AL
	POP AX
	Cls
	Switch psOff
	ret
endp

_InitGraphics proc
	MOV CS:[cursor],0	;Reset the cursor position
	MOV CS:[ioState],ioGraphic
	PUSH AX
	MOV AL,0D3h
	OUT 02,AL
	POP AX
	Cls
	Switch psOn
	ret
endp

_Cls proc
	PUSH AX CX ES DI	;Save the registers
	XOR AX,AX		;and clear
	MOV CX,512*4		;the screen
	PUSH 1A20h		;by writing with
	POP ES			;0s
	XOR DI,DI		;512
	REP STOSW		;times now
	MOV word ptr CS:[cursor],0
	call _drawMouse
	POP DI ES CX AX 	;then resore registers.
	ret
endp

_scrollUp proc
	PUSH ES DI DS SI AX CX
	MOV AX,01A20h		;Set addresses of video buffe
	MOV ES,AX
	MOV DS,AX
	MOV DI,63
	MOV SI,55
	XOR AX,AX
	STD			;Move reversen since we start in lower right corner
	@@movecol:
	MOV CX,28		
	REP MOVSW		;And move the fist 7 lines
	MOV CX,4
	REP STOSW		;and then clear the last line
	ADD DI,128
	ADD SI,120
	CMP SI,16*64		;Do this until all columns are moved up
	JBE @@movecol
	CLD			;And DON'T forgett to reset the direction ;)
	POP CX AX SI DS DI ES	;Then restore th registers
	ret			;and return to caller
endp